@@ -388,10 +388,19 @@ class Recognizer(WebSocketClient):
     def opened(self):
         """Open websocket"""
         svc_logger.info("websocket opened")
+        if self.__session_started_event.is_set:
+            evt = {"session_id": self.session_id, "status": 0, "message": "recognition started"}
+            evt_res = EventResults(evt)
+            self.__session_started_event.callback(evt_res)
 
     def closed(self, code, reason):
         """Close websocket"""
         svc_logger.info(f"Closed down: {code}, {reason}")
+        if self.__session_stopped_event.is_set:
+            evt = {"session_id": self.session_id, "status": code, "messge": reason}
+            evt_res = EventResults(evt)
+            self.__session_stopped_event.callback(evt_res)
+        self.close()
 
     def send_thread(self):
         """Send data to websocket"""
@@ -403,20 +412,15 @@ class Recognizer(WebSocketClient):
                         self.send(buf, binary=True)
             except KeyboardInterrupt:
                 self.stop_continuous_recognition_async()
-        else:
-            if self.__canceled_event.is_set:
-                evt = {"session_id": self.session_id}
-                evt_res = EventResults(evt)
-                self.__canceled_event.callback(evt_res)
-            self.send("EOS")
-            self.close()
+        self.send("EOS")
 
     def received_message(self, message) -> None:
         """Receive result from websocket"""
         try:
             evt = json.loads(str(message))
-            if evt['status'] == 0:
+            if 'id' in evt:
                 self.session_id = evt['id']
+            if evt['status'] == 0:
                 if 'result' in evt:
                     if evt['result']['final']:
                         if self.__recognized_event.is_set:
@@ -430,21 +434,26 @@ class Recognizer(WebSocketClient):
                             evt_res = EventResults(evt)
                             self.__recognizing_event.callback(evt_res)
             else:
-                errmsg = "Received error from server: "
                 if 'message' in evt:
-                    errmsg += f"{evt['message']}"
-                raise RuntimeError(errmsg)
+                    evt['message'] = "Received error from server: " + f"{evt['message']}"
+                else:
+                    evt['message'] = "Got error from server."
+                raise RuntimeError(evt)
         except Exception as e:
-            svc_logger.error(f"Exception in receive of continuous_recognition_async: {e}")
+            svc_logger.error(f"Exception in receive_message: {evt}")
+            if self.__canceled_event.is_set:
+                evt_res = EventResults(evt)
+                self.__canceled_event.callback(evt_res)
             self.stop_continuous_recognition_async()
 
     def stop_continuous_recognition_async(self):
         """Stop recognition"""
         self.is_running = False
+        self.__input_stream.close()
 
     def stop_continuous_recognition(self):
         """Stop recognition"""
-        self.is_running = False
+        self.stop_continuous_recognition_async()
         self.__recognizer_semaphore.release()
 
     @property
@@ -568,6 +577,7 @@ class EventResults():
         self.segment_start = None
         self.segment_length = None
         self.total_length = None
+        self.message = None
 
         if "status" in evt:
             self.status = evt.get("status")
@@ -583,6 +593,8 @@ class EventResults():
             self.segment_length = evt.get("segment-length")
         if 'total-length' in evt:
             self.total_length = evt.get("total-length")
+        if 'message' in evt:
+            self.message = evt.get("message")
 
     def __str__(self):
         """Return event info"""
@@ -590,7 +602,8 @@ class EventResults():
             f"status={self.status}, "
             f"segment={self.segment}, "
             f"result=({self.result}), "
-            f"session_id={self.session_id}"
+            f"session_id={self.session_id}, "
+            f"message={self.message}"
         )
 
 
